*! 2.3.2  26feb2014
program pearsonx2
	version 13.0

	if "`e(cmd)'" == "" {
		error 301
	}

	local default_eps .01
	// option -detail- is not documented
	syntax [, Detail Eps(real `default_eps') Table ]

	if !inrange(`eps', 0, 0.5) {
		local eps `default_eps'
	}

	// only frequency weights support
	// no clustering
	if "`e(wtype)'" != "" & "`e(wtype)'" != "fweight" {
		dis as err "pearsonx2 does not support `e(wtype)' weights"
		exit 198
	}
	if "`e(vcetype)'" != "" {
		dis as txt "(pearsonx2 ignores robustness/clustering)"
	}

/*
	-----------------------------------------------------------------
	Driver: Computes Pearson's X2 and Windmeijer's normalization

	The driver command should set:

	r(N)       number of observations used
	r(N_drop)  number of observations ignored
	r(X2)      Pearson's X2
	r(meanX2)  estimate of (asymptotic) mean of X2
	r(varX2)   estimate of (asymptotic) variance of X2
	r(chi2)    Windmeijer's normalization of X2

	Note: In clogit, we define N and N_drop as the number
	of groups, not the number records/obs.
   	-----------------------------------------------------------------
*/

	if inlist("`e(cmd)'","logit","logistic","probit","cloglog") {
		local PearsonCmd Binary
	}
	else if "`e(cmd)'" == "clogit" {
		TestClogit   /* check: one success per group */
		local PearsonCmd Clogit
	}
	else if "`e(cmd)'" == "mlogit" {
		local PearsonCmd Mlogit
	}
	else {
		dis as err "pearsonx2 does not support `e(cmd)'"
		exit 301
	}
	quietly `PearsonCmd' `eps'

	if r(N) == 0 {
		dis as txt "(no observations left to compute X2)"
		exit 2000
	}
	if r(varX2) == . {
		dis as txt "(negative approximate variance, test skipped)"
		exit
	}

	Display `eps' `detail'

	if "`table'" != "" {
		SensitivityTable `PearsonCmd'
	}
end


/* =========================================================================
   BINARY CHOICE MODELS.
   =========================================================================

   Windmeijer (Statistica Neerlandica 1990: page 71) has shown that, under
   regularity conditions that imply Liaponov's CLT,

      n^(-1/2) (X2 - n) / s_n ---> N(0,1)   (convergence in distribution)

   where

              1  n   (1-2p_i)^2
      s_n^2 = - SUM  -----------   - v_n' inv(I_n) v_n
              n i=1  p_i*(1-p_i)
              ----- term 1 -----     ---- term 2 -----

                                      1  n    f_i^2
      I_n = mean Fisher Information = - SUM ---------- x_i x_i'
                                      n i=1 p_i(1-p_i)
            1  n    1-2p_i
      v_n = - SUM --------- f_i x_i
            n i=1 p_i(1-p_i)

      f_i = F'(xb_i)        <------ note this depends on the link function!

========================================================================== */

program Binary, rclass
	args eps

	tempvar f p pq touse xb w
	tempname beta term1 term2 v V Omega

	if "`e(wtype)'" != "" {
		// at this point only fweights are possible
		local wght `"[`e(wtype)'`e(wexp)']"'
	}

// determine sample for which X2 is to be computed

	gen byte `touse' = e(sample)
	predict double `p' if `touse', p

	// discard observations with p < eps or p > 1-eps
	// note: eps < 0.5 is ensured
	summ `touse' if !inrange(`p',`eps',1-`eps') `wght' , meanonly
	return scalar N_drop = r(sum)
	if return(N_drop) > 0 {
		replace `touse' = `touse' & inrange(`p',`eps',1-`eps')
	}
	summ `touse' `wght' , meanonly
	return scalar N = r(sum)  /* #obs used */

// Compute X2 and varX2

	gen double `pq' =  `p' * (1-`p') if `touse'
	gen double `w'  = ((`e(depvar)'!=0)-`p')^2 / (`pq') if `touse'
	summ `w' if `touse' `wght', meanonly
	return scalar X2 = r(sum)             /* unaggregated X2-statistic */
	drop `w'

	// Compute first term (term1) in (71:5)
	gen double `w' = (1-2*`p')^2 / (`pq') if `touse'
	summ `w' if `touse' `wght', meanonly
	scalar `term1' = r(mean)
	drop `w'

	matrix `beta' = e(b)
	local xvars : colnames(`beta')
	local xvars : subinstr local xvars "_cons" "", word all count(local nch)
	if `nch' > 0 {
		tempvar `cons'
		local xvars `xvars' `cons'
	}

	// evaluate f = F'(lp)
	if "`e(cmd)'"=="logit" | "`e(cmd)'"=="logistic" {
		gen double `f' = `pq' if `touse'
	}
	else if "`e(cmd)'" == "probit" {
		predict double `xb' if `touse', xb
		gen double `f' = normalden(`xb') if `touse'
	}
	else if "`e(cmd)'" == "cloglog" {
		predict double `xb' if `touse', xb
		* !! should we use appoximation if xb very large? See cloglog.
		gen double `f' = exp(-`xb') if `touse'
		replace `f' = `f' * exp(-`f') if `touse'
		replace `f' = 0 if missing(`f') & `touse'
	}

	// exp (71:3): Omega = n * Omega_n
	// we cannot use e(V) as we may have dropped obs
	gen double `w' = (`f'^2) / (`pq')
	if `"`wght'"' != "" {
		local ewexp `"`e(wexp)'"'    /* e(wexp) contains "= expression" */
		gettoken tok wexp : ewexp
		replace `w' = `w' * `wexp'
	}
	matrix accum  `Omega' = `xvars' [iw = `w']  if `touse' , nocons
	drop `w'

	// exp (71:5): v = n * v_n
	gen double `w'  = ((1-2*`p')*`f') / `pq' if `touse'
	matrix vecacc `v' = `w' `xvars'  if `touse' `wght' , nocons
	matrix `V' = `v' * syminv(`Omega') * `v''
	scalar `term2' = `V'[1,1] / return(N)

	// noi di "term1 " `term1'
	// noi di "term2 " `term2'

	// form mean(X2) and var(X2)
	return scalar meanX2 = return(N)
	return scalar varX2  = return(N) * (`term1' - `term2')

	// AVAR is not ensured to be positive in finite samples (!)
	if return(varX2) < 0 {
		return scalar varX2 = .
	}
	// compute Windmeijer's chi2(1) test statistic H (page 72)
	return scalar chi2 = (return(X2)-return(meanX2))^2 / return(varX2)
end


/* =========================================================================
   CONDITIONAL LOGIT MODEL
   =========================================================================

   We only consder the case with one success (1) per group. See Windmeijer
   (Statistica Neerlandica 1994): section 5, page 279.

     n^(-1/2) (X2 - n*m) / s_n -> N(0,1)

             1  n   m  ( 1-2p_ij          m            )
     s_n^2 = - SUM SUM ( ------- q_ij  - SUM q_ij*q_ih ) - v_n' inv(I_n) v_n
             n i=1 j=1 (   p_ij          h<>j          )
             ------------------ term 1 -----------------   ---- term 2 -----

     q_ij  = (1-2p_ij)/(1-p_ij)

     I_n = mean Fisher Information

             1  n   m                 m
     v_n   = - SUM SUM q_ij ( z_ij - SUM p_ir z_ir)
             n i=1 j=1               r=1

   I (jw) ran through the proof, it seems to generalize without problem to
   the case with m depending on i.

   Note that term-1 in s_n^2 differs from Windmeijer in the summation
   over h. The expression in SN and similarly in Windmijer's thesis turn
   out to be incorrect. Windmeijer (private communication with JW,
   May 2001) confirmed the mistake.

========================================================================== */

program TestClogit, sortpreserve
	tempvar cy

	egen `cy' = sum(`e(depvar)'!=0) if e(sample) , by(`e(group)')
	qui assert `cy' == 1 if e(sample)
	if _rc {
		dis as err /*
		 */ "pearsonx2 only supports clogit with one positive outcome per group"
		exit 498
	}
end

program Clogit, rclass sort
	args eps

	tempname beta x2 term1 term2 vn V Omega N0
	tempvar cq group minp p pq q touse x

	if "`e(wtype)'" != "" {
		// at this point only fweights are possible
		// it is assumed that the weights are constant within groups
		local wght `"[`e(wtype)'`e(wexp)']"'
	}

/*
	determine sample for which X2 is to be computed
	we exclude cases (groups) with some p < eps

	no test for validity eps needed at this stage
*/

	gen byte `touse' = e(sample)
	predict double `p' if `touse', p

	// count number of groups
	bys `touse' `e(group)' : gen byte `group' = _N==_n if `touse'
	summ `group' `wght', meanonly
	scalar `N0' = r(sum)

	// drop cases with minimum p < eps
	// alternative approach: truncation at eps
	bys `touse' `e(group)' (`p') : gen `minp' = `p'[1] if `touse'
	summ `group' if `minp'<`eps' `wght' , meanonly
	return scalar N_drop = r(sum)
	return scalar N = `N0' - return(N_drop)   /* number of groups */
	if return(N_drop) > 0 {
		replace `touse' = 0 if `touse' & (`minp'<`eps')
	}
	if return(N) == 0 { 
		exit 
	}
	drop `minp' `group'

// compute X2

	gen double `pq' = `p' * (1-`p') if `touse'
	gen double `x' = (((`e(depvar)'!=0)-`p')^2)/(`pq') if `touse'
	summ `x' if `touse' `wght' , meanonly
	return scalar X2 = r(sum)        /* X2-statistic */
	return scalar meanX2 = r(N)      /* asymptotic mean of X2 */
	drop `x'

// compute var(X2)

	gen double `q' = (1-2*`p')/(1-`p') if `touse'

	/* first term of sigma_n^2, part 1

	   In the SN article, summing over q(ij) occurs within the summation
	   over h. This was a mistake.
	*/

	gen double `x' = `q' * (1-2*`p')/`p' if `touse'
	summ `x' if `touse' `wght', meanonly
	scalar `term1' = r(sum)
	drop `x'

	/* first term of sigma_n^2, part 2

	   we use the algebraic re-formulation
	   sum_j sum_{h<>j} q_ij q_ih = 2 sum_j q_ij sum_{h<j} q_ih
	*/

	bys `touse' `e(group)' : gen double `cq' = sum(`q'[_n-1]) if `touse'
	replace `cq' = 2 * `cq' * `q' if `touse'
	summ `cq' if `touse' `wght', meanonly
	scalar `term1' = (`term1' - r(sum)) / return(N)
	drop `cq'

	/* second term of s_n^2

	   this corrects for the fact that the b's are estimated rather
	   than being known.

   	   produce zcvars, the within-group p-weighted centered versions
   	   of the xvars
	*/

	matrix `beta' = e(b)
	local varlist : colnames(`beta')
	tempvar s
	local i 0
	foreach x of local varlist {
		local i = `i'+1
		tempvar zc`i'
		bys `touse' `e(group)' : gen double `s'     = sum(`p'*`x') if `touse'
		bys `touse' `e(group)' : gen double `zc`i'' = `x'-`s'[_N]  if `touse'
		local zcvars `zcvars' `zc`i''
		drop `s'
	}

// compute Omega and vn, both a multiplicative factor return(N) too large

	if "`wght'" != "" {
		tempvar iw
		local ewexp `"`e(wexp)'"' /* e(wexp) contains "= expression" */
		gettoken tok wexp : ewexp
		gen double `iw' = `p' * `wexp'
	}
	else {
		local iw `p'
	}
	matrix accum `Omega' = `zcvars' [iw = `iw'] if `touse', nocons
	matrix vecacc `vn'   = `q' `zcvars' if `touse' `wght' , nocons
	matrix `V' = `vn' * syminv(`Omega') * `vn''
	scalar `term2' = `V'[1,1] / return(N)
	// noi matrix list `Omega', title(clogit:Omega)
	// noi matrix list `vn', title(clogit:vn)

	// form var(X2)
	// noi di "term1 " `term1'
	// noi di "term2 " `term2'
	return scalar varX2 = return(N) * (`term1' - `term2')
	if return(varX2) < 0 {
		return scalar varX2 = .
	}

	// compute Windmeijer's chi2(1) test statistic
	// it is asymptotically Chi2(1df) under H0
	return scalar chi2 = (return(X2)-return(meanX2))^2 / return(varX2)
end

/* =========================================================================
   MULTINOMIAL LOGISTIC REGRESSION
   =========================================================================
   Windmeijer  (1994: ..) has shown that

     n^(-1/2) (X2 - n*m) / s_n --> N(0,1)

             1  n   m  ( 1-2p_ij          m            )
     s_n^2 = - SUM SUM ( ------- q_ij  - SUM q_ij*q_ih ) - v_n' inv(I_n) v_n
             n i=1 j=1 (   p_ij          h<>j          )
               ----------------- term 1 ----------------   ---- term 2 -----

     q_ij  = (1-2p_ij)/(1-p_ij)

     I_n = mean Fisher Information

     v_n = (v_n1, ... v_nm)  -- note: omit term for baseline category

            1  n   m                 m
     v_nr = - SUM SUM ( q_ij - p_ir SUM q_ij ) z_i
            n i=1 j=1               j=1

   Note that term-1 in s_n^2 differs from Windmeijer in the summation over
   h.  The expression in SN and similarly in Windmijer's thesis turn out
   to be incorrect.  Windmeijer (private communication with JMW, May 2001)
   confirmed the mistake.

========================================================================= */

program Mlogit, rclass
	args eps

	tempname b N0 term1 term2 vn V gradient out
	tempvar s sumq touse y 

	// extract info from e()
    local base `e(ibaseout)'
	local m      = e(k_eq)
	matrix `out' = e(out) 
	matrix `b'   = e(b)
	matrix `V'   = e(V)
	gen byte `touse' = e(sample)

	// predictions p1..pm
	forvalues i = 1/`m' {
		tempvar  p`i'
		local plist `plist' `p`i''
	}
	predict double `plist' // predictions for all categories

	if "`e(wtype)'" != "" {
		// at this point only fweights are possible
		local wght `"[`e(wtype)'`e(wexp)']"'
	}

// exclude obs with some p<eps

	summ `touse' `wght', meanonly
	scalar `N0' = r(sum)
	forvalues i = 1/`m' {
		replace `touse' = `touse' & `p`i'' > `eps'
	}
	summ `touse' `wght', meanonly
	return scalar N = r(sum)
	return scalar N_drop = `N0' - return(N)
	if return(N) == 0 {
		exit 2000
	}

// compute Pearson's X2 (page 271: eq 1)

	gen double `s' = 0
	forvalues j = 1/`m' {
		gen byte `y' = float(`e(depvar)') == float(`out'[1,`j'])
		replace `s' = `s' + ((`y'-`p`j'')^2)/(`p`j''*(1-`p`j'')) if `touse'
		drop `y'
	}
	summ `s' if `touse' `wght'
	return scalar X2 = r(sum)
	drop `s'

	// term1 of expression for sigma^2 (page 274, line 2)
	gen double `sumq' = 0 if `touse'
	forvalues j = 1/`m' {
		tempvar q`j'
		gen double `q`j'' = (1-2*`p`j'')/(1-`p`j'') if `touse'
		replace `sumq' = `sumq' + `q`j'' if `touse'  /* used in eq (274:4) */
	}
	gen double `s' = 0 if `touse'
	forvalues j = 1/`m' {
		// in contrast with the expression in Windmeijer '94,
		// this should be outside the h-loop...
		replace `s' = `s' + `q`j''*(1-2*`p`j'')/`p`j'' if `touse'
		forvalues h = 1/`m' {
			if `j'!=`h' {
				replace `s' = `s' - `q`j''*`q`h'' if `touse'
			}
		}
	}
	summ `s' if `touse' `wght'
	scalar `term1' = r(mean)
	// dis as err "{txt}term1 " `term1'

	// extract varlist
	matrix `gradient' = e(gradient)
	local varnames : colnames `gradient'
	DropDup xvars : "`varnames'"
	local nvars : word count `xvars'
	// _cons -> `cons'
	local xvars : subinstr local xvars "_cons" "", word count(local nch)
	if `nch' > 0 {
		tempvar cons
		gen byte `cons' = 1
		local xvars `xvars' `cons'
	}

	// compute vn (274: eq 4)
	local nb = colsof(`V')
	matrix `vn' = J(1,`nb',0)
	tempvar z
	local it 0
	forvalues j = 1/`m' {
		if `j'!=`e(ibaseout)' {
			foreach x of local xvars {
				gen double `z' = (`q`j''-`sumq'*`p`j'')*`x' if `touse'
				summ `z' if `touse' `wght', meanonly
				local it = `it' + 1
				matrix `vn'[1,`it'] = r(mean)
				drop `z'
			}
		}
	}
	//noi matrix list `vn', title(mlogit:vn)

	// build Omega manually (Note we can't use e(V) due to dropped obs)
	if "`e(wtype)'" != "" {
		local ewexp `"`e(wexp)'"' /* e(wexp) contains "= expression" */
		gettoken tok iw : ewexp
	}
	else {
		local iw 1
	}

	tempname Omega
	matrix `Omega' = J(`nb',`nb',0)
	local ind0 1
	forvalues j = 1/`m' {
		if `j'!=`e(ibaseout)' {
			local ind`j' = `ind0'
			local ind0 = `ind`j'' + `nvars'
			matrix accum `V' = `xvars' [iw = `p`j''*(1-`p`j'')*(`iw')] if `touse', nocons
			matrix `Omega'[`ind`j'',`ind`j''] = `V'
		}
	}
	forvalues j = 1/`m' {
		if `j'!=`e(ibaseout)' {
			local jmin1 = `j'-1
			forvalues h = 1/`jmin1' {
				if `h' != `base' {
					matrix accum `V' = `xvars' [iw = `p`j''*`p`h''*(`iw')] if `touse', nocons
					matrix `Omega'[`ind`j'',`ind`h''] = - `V'
					matrix `Omega'[`ind`h'',`ind`j''] = - `V''
				}
			}
		}
	}

	/*
		matrix list `Omega', title(mlogit:Omega)
		tempname Iomega V
		matrix `Iomega' = syminv(`Omega')
		matrix list `Iomega', title(mlogit:Iomega)
		matrix `V' = e(V)
		matrix list `V', title(mlogit:e(V))
		noi dis as err _n "mreldif :: " mreldif(`V',`Iomega') _n
	*/
	
	matrix `V' = `vn' * syminv(`Omega') * `vn''
	scalar `term2' = `V'[1,1] * return(N)
	// dis as err "{txt}term2 " `term2'

	// form mean(X2) and var(X2)
	return scalar meanX2 = return(N) * `m'
	return scalar varX2  = return(N) * (`term1' - `term2')
	// AVAR is not ensured to be positive in finite samples (!)
	if return(varX2) < 0 {
		return scalar varX2 = .
	}

	// compute Windmeijer's chi2(1) test statistic H (page 72)
	return scalar chi2 = (return(X2)-return(meanX2))^2 / return(varX2)
end


// =========================================================================
// utility commands
// =========================================================================

/* DropDup newlist : list
   drops all duplicate tokens from list -- copied from hausman.ado
*/
program DropDup
	args newlist  /*  name of macro to store new list
	 */  colon    /*  ":"
	 */  list     /*  list with possible duplicates */

	gettoken token list : list
	while "`token'" != "" {
		local fixlist `fixlist' `token'
		local list : subinstr local list "`token'" "", word all
		gettoken token list : list
	}

	c_local `newlist' `fixlist'
end

program Display
	args eps detail

	dis _n as txt "Pearson-Windmeijer goodness-of-fit test after " /*
	   */ as res "`e(cmd)' `e(depvar)'" _n

	local col 31
	if r(N_drop) > 0 {
		dis as txt "{ralign `col':number of excluded obs =}" /*
		 */ as res %9.0g r(N_drop)
		dis as txt "{ralign `col':theshold for exclusion eps =}" /*
		 */ as res %9.0g `eps'
		dis as txt "{ralign `col':number of obs used =}" /*
		 */ as res %9.0g r(N)
	}
	else {
		dis as txt "{ralign `col':number of observations =}" /*
		 */ as res %9.0g r(N)
	}

	dis as txt "{ralign `col':Pearson's X2 (ungrouped) =}" /*
	 */ as res %9.2f r(X2)
	if "`detail'" != "" {
		dis as txt "{ralign `col':asympt mean X2 =}" /*
		 */ as res %9.2f r(meanX2)
		dis as txt "{ralign `col':asympt Std. Dev. X2 =}" /*
		 */ as res %9.2f sqrt(r(varX2))
	}
	dis as txt "{ralign `col':Windmeijer's H = norm(X2) =}" /*
	 */ as res %9.2f r(chi2)
	dis as txt "{ralign `col':Prob > chi2(1) =}" /*
	 */ as res %9.4f chi2tail(1,r(chi2)) _n
end

program SensitivityTable
	args cmd

	dis as txt "{hline 11}{c TT}{hline 52}"
	dis "      eps  {c |}   Obs        X2      se(X2)         H         p"
	dis "{hline 11}{c +}{hline 52}"

	local eps = 0.1
	while `eps' > 0 {
		capt `cmd' `eps'
		if !_rc {
			#del ;
			dis as txt %9.8f `eps' " {c |} "
			    as res %5.0f  r(N)
			           %11.2f r(X2)
			           %11.2f sqrt(r(varX2))
			           %11.2f r(chi2)
			           %10.4f chi2tail(1,r(chi2)) ;
			#del cr
		}
		else {
			dis as txt %9.8f `eps' " {c |} <undefined> "
		}
		local eps = cond(r(N_drop)>0, cond(`eps'>1e-8,`eps'/10, 0), 0)
	}
	dis as txt "{hline 11}{c BT}{hline 52}"
	dis "{ralign 64:{it:All obs with some p<eps are ignored in computing the test}}"
	dis
end

exit

HISTORY

2.3.2   upgraded to Stata version 13 (on Paul Allison's request)
2.3.0   added support for fweights
